@@ -22,7 +22,6 @@
 #include <string.h>
 #include <stdarg.h>
 
-
 typedef struct _BablFindFish BablFindFish;
 
 typedef struct _BablFindFish
@@ -48,14 +47,6 @@ static int
 find_memcpy_fish (Babl *item,
                   void *data);
 
-static int
-each_babl_fish_destroy (Babl *babl,
-                        void *data);
-
-
-/* ====================================== */
-
-
 static int
 find_fish_path (Babl *item,
                 void *data)
@@ -145,14 +136,6 @@ babl_fish_get_id (const Babl *source,
   return id;
 }
 
-BablDb *
-babl_fish_db (void)
-{
-  if (!db)
-    db = babl_db_init ();
-  return db;
-}
-
 Babl *
 babl_fish (const void *source,
            const void *destination)
@@ -278,16 +261,4 @@ babl_fish (const void *source,
   }
 }
 
-
-
-static int
-each_babl_fish_destroy (Babl *babl,
-                        void *data)
-{
-  if (babl->class_type == BABL_FISH_PATH)
-    babl_list_destroy (babl->fish_path.conversion_list);
-  babl_free (babl);
-  return 0;  /* continue iterating */
-}
-
 BABL_CLASS_MINIMAL_IMPLEMENT (fish);
