@@ -24,15 +24,19 @@
 #include "babl-internal.h"
 #include "babl-db.h"
 
-static int
-each_babl_format_destroy (Babl *babl,
-                          void *data)
+static int babl_format_destruct (void *babl)
 {
-  if (babl->format.from_list)
-    babl_list_destroy (babl->format.from_list);
-  babl_free (babl);
-
-  return 0;  /* continue iterating */
+  BablFormat *format = babl;
+  if (format->image_template != NULL)
+    {
+      babl_set_destructor (format->image_template, NULL);
+      /* with no destructor set, the circular reference is no problem */
+      babl_free (format->image_template);
+      format->image_template = NULL;
+    }
+  if (format->from_list)
+    babl_free (format->from_list);
+  return 0;
 }
 
 static Babl *
@@ -47,7 +51,7 @@ format_new (const char     *name,
 {
   Babl *babl;
 
-  /* i is desintation position */
+  /* i is destination position */
   int i, j, component_found = 0;
   for (i = 0; i < model->components; i++)
     {
@@ -75,6 +79,7 @@ format_new (const char     *name,
                       sizeof (BablType *) * (components) +
                       sizeof (int) * (components) +
                       sizeof (int) * (components));
+  babl_set_destructor (babl, babl_format_destruct);
 
   babl->format.from_list = NULL;
   babl->format.component = (void *) (((char *) babl) + sizeof (BablFormat));
@@ -111,14 +116,13 @@ format_new (const char     *name,
 }
 
 
-static char buf[512] = "";
-
 static char *
 create_name (BablModel      *model,
              int             components,
              BablComponent **component,
              BablType      **type)
 {
+  char            buf[512] = "";
   char           *p = &buf[0];
   int             i;
   int             same_types = 1;
@@ -162,7 +166,7 @@ create_name (BablModel      *model,
   if (same_types)
     {
       sprintf (p, "%s", first_type->instance.name);
-      return buf;
+      return babl_strdup (buf);
     }
 
   i = components;
@@ -177,7 +181,85 @@ create_name (BablModel      *model,
       component++;
       type++;
     }
-  return buf;
+  return babl_strdup (buf);
+}
+
+
+static char *
+ncomponents_create_name (Babl *type,
+                         int   components)
+{
+  char buf[512];
+  sprintf (buf, "%s[%i] ", type->instance.name, components);
+  return babl_strdup (buf);
+}
+
+Babl *
+babl_format_n (Babl *btype,
+               int   components)
+{
+  int            i;
+  Babl          *babl;
+  int            id         = 0;
+  int            planar     = 0;
+  BablModel     *model      = (BablModel *)babl_model ("Y");
+  BablComponent *component [components];
+  BablSampling  *sampling  [components];
+  BablType      *type      [components];
+  char          *name       = NULL;
+
+  for (i = 0; i<components; i++)
+    {
+      component[i] = model->component[0];
+      type[i] = &btype->type;
+      sampling[i] = (BablSampling *) babl_sampling (1, 1);
+    }
+
+  name = ncomponents_create_name (btype, components);
+  babl = babl_db_exist (db, id, name);
+  if (babl)
+    {
+      /* There is an instance already registered by the required id/name,
+       * returning the preexistent one instead.
+       */
+      babl_free (name);
+      return babl;
+    }
+
+  babl = format_new (name,
+                     id,
+                     planar, components, model,
+                     component, sampling, type);
+
+  babl_db_insert (db, babl);
+  babl_free (name);
+  return babl;
+}
+
+static int
+is_format_duplicate (Babl           *babl,
+                     int             planar,
+                     int             components,
+                     BablModel      *model,
+                     BablComponent **component,
+                     BablSampling  **sampling,
+                     BablType      **type)
+{
+  int i;
+
+  if (babl->format.planar     != planar     ||
+      babl->format.components != components ||
+      babl->format.model      != model)
+    return 0;
+
+  for (i = 0; i < components; i++)
+    {
+      if (babl->format.component[i] != component[i] ||
+          babl->format.sampling[i]  != sampling[i]  ||
+          babl->format.type[i]      != type[i])
+        return 0;
+    }
+  return 1;
 }
 
 Babl *
@@ -274,7 +356,7 @@ babl_format_new (void *first_arg,
 
       else if (!strcmp (arg, "name"))
         {
-          name = va_arg (varg, char *);
+          name = babl_strdup (va_arg (varg, char *));
         }
 
       else if (!strcmp (arg, "packed"))
@@ -303,11 +385,20 @@ babl_format_new (void *first_arg,
     name = create_name (model, components, component, type);
 
   babl = babl_db_exist (db, id, name);
+  if (id && !babl && babl_db_exist (db, 0, name))
+    babl_fatal ("Trying to reregister BablFormat '%s' with different id!", name);
+
   if (babl)
     {
       /* There is an instance already registered by the required id/name,
-       * returning the preexistent one instead.
+       * returning the preexistent one instead if it doesn't differ.
        */
+      if (!is_format_duplicate (babl, planar, components, model,
+                                component, sampling, type))
+        babl_fatal ("BablFormat '%s' already registered "
+                    "with different content!", name);
+
+      babl_free (name);
       return babl;
     }
 
@@ -316,10 +407,8 @@ babl_format_new (void *first_arg,
                      planar, components, model,
                      component, sampling, type);
 
-  /* Since there is not an already registered instance by the required
-   * id/name, inserting newly created class into database.
-   */
   babl_db_insert (db, babl);
+  babl_free (name);
   return babl;
 }
 
