@@ -40,15 +40,16 @@
 #include "babl.h"
 #define _BABL_INTERNAL_H
 
-#include "babl-class.h"
 #include "babl-classes.h"
 #include "babl-introspect.h"
+#include "babl-class.h"
 #include "babl-list.h"
 #include "babl-hash-table.h"
 #include "babl-db.h"
 #include "babl-ids.h"
 #include "babl-util.h"
 #include "babl-memory.h"
+#include "babl-mutex.h"
 #include "babl-cpuaccel.h"
 
 /* redefining some functions for the win32 platform */
@@ -79,17 +80,13 @@ Babl   * babl_extender                  (void);
 void     babl_set_extender              (Babl           *new_extender);
 
 Babl   * babl_extension_quiet_log       (void);
+void     babl_extension_deinit          (void);
 
-long     babl_fish_process              (Babl           *babl,
-                                         void           *source,
-                                         void           *destination,
-                                         long            n);
 long     babl_fish_reference_process    (Babl           *babl,
                                          BablImage      *source,
                                          BablImage      *destination,
                                          long            n);
 
-BablDb * babl_fish_db                   (void);
 Babl   * babl_fish_reference            (const Babl     *source,
                                          const Babl     *destination);
 Babl   * babl_fish_simple               (BablConversion *conversion);
@@ -97,10 +94,6 @@ void     babl_fish_stats                (FILE           *file);
 Babl   * babl_fish_path                 (const Babl     *source,
                                          const Babl     *destination);
 
-long     babl_fish_path_process         (Babl           *babl,
-                                         void           *source,
-                                         void           *destination,
-                                         long            n);
 int      babl_fish_get_id               (const Babl     *source,
                                          const Babl     *destination);
 
@@ -132,7 +125,7 @@ static void hack_hack (void)
 /**** LOGGER ****/
 #include <stdarg.h>
 
-void babl_backtrack (void);
+int babl_backtrack (void);
 
 static inline void
 real_babl_log (const char *file,
@@ -238,25 +231,18 @@ babl_fatal (const char *format, ...)
 )
 
 extern int   babl_hmpf_on_name_lookups;
+extern BablMutex *babl_format_mutex;
+
+#define BABL_DEBUG_MEM 0
+#if BABL_DEBUG_MEM
+extern BablMutex *babl_debug_mutex;
+#endif
 
 const char  *babl_class_name       (BablClassType klass);
 void         babl_internal_init    (void);
 void         babl_internal_destroy (void);
 
 
-#ifndef BABL_INIT_HOOK
-#define BABL_INIT_HOOK
-#endif
-#ifndef BABL_PRE_INIT_HOOK
-#define BABL_PRE_INIT_HOOK
-#endif
-#ifndef BABL_DESTROY_HOOK
-#define BABL_DESTROY_HOOK
-#endif
-#ifndef BABL_DESTROY_PRE_HOOK
-#define BABL_DESTROY_PRE_HOOK
-#endif
-
 /* this template is expanded in the files including babl-internal.h,
  * generating code, the declarations for these functions are found in
  * the BABL_CLASS expansions done in babl.h as well, thus babl.h needs
@@ -264,22 +250,13 @@ void         babl_internal_destroy (void);
  */
 
 #define BABL_CLASS_MINIMAL_IMPLEMENT(klass)                   \
-void                                                          \
-babl_##klass##_class_init (void)                              \
+                                                              \
+BablDb *                                                      \
+babl_##klass##_db (void)                                      \
 {                                                             \
-  BABL_PRE_INIT_HOOK;                                         \
   if (!db)                                                    \
     db=babl_db_init ();                                       \
-  BABL_INIT_HOOK;                                             \
-}                                                             \
-                                                              \
-void                                                          \
-babl_##klass##_class_destroy (void)                           \
-{                                                             \
-  BABL_DESTROY_PRE_HOOK;                                      \
-  babl_db_each (db,each_babl_##klass##_destroy, NULL);        \
-  babl_db_destroy (db);                                       \
-  BABL_DESTROY_HOOK;                                          \
+  return db;                                                  \
 }                                                             \
                                                               \
 void                                                          \
@@ -320,7 +297,7 @@ babl_##klass##_from_id (int id)                               \
       babl_fatal ("%s(%i): not found", G_STRFUNC, id);        \
     }                                                         \
   return babl;                                                \
-}
+} \
 
 #define BABL(obj)  ((Babl*)(obj))
 
