@@ -23,35 +23,30 @@
 #include "babl-internal.h"
 #include "babl-db.h"
 
+static Babl *construct_double_format (Babl *model);
 
 static int
-each_babl_model_destroy (Babl *babl,
-                         void *data)
+babl_model_destroy (void *data)
 {
+  Babl *babl = data;
   if (babl->model.from_list)
-    babl_list_destroy (babl->model.from_list);
-  babl_free (babl);
-  return 0;  /* continue iterating */
+    babl_free (babl->model.from_list);
+  return 0; 
 }
 
-static char buf[512] = "";
-
-static const char *
-create_name (const char     *name,
-             int             components,
-             BablComponent **component)
+static char *
+babl_model_create_name (int             components,
+                        BablComponent **component)
 {
-  char *p = buf;
+  char *p = NULL;
 
-  if (name)
-    return name;
   while (components--)
     {
-      sprintf (p, (*component)->instance.name);
-      p += strlen ((*component)->instance.name);
+      p = babl_strcat(p, (*component)->instance.name);
       component++;
     }
-  return buf;
+
+  return p;
 }
 
 static Babl *
@@ -65,6 +60,7 @@ model_new (const char     *name,
   babl = babl_malloc (sizeof (BablModel) +
                       sizeof (BablComponent *) * (components) +
                       strlen (name) + 1);
+  babl_set_destructor (babl, babl_model_destroy);
   babl->model.component = (void *) (((char *) babl) + sizeof (BablModel));
   babl->instance.name   = (void *) (((char *) babl->model.component) + sizeof (BablComponent *) * (components));
 
@@ -78,16 +74,35 @@ model_new (const char     *name,
   return babl;
 }
 
+static int
+is_model_duplicate (Babl *babl, int components, BablComponent **component)
+{
+  int   i;
+
+  if (babl->model.components != components)
+    return 0;
+
+  for (i = 0; i < components; i++)
+    {
+      if (babl->model.component[i] != component[i])
+        return 0;
+    }
+
+  return 1;
+}
+
+
 Babl *
 babl_model_new (void *first_argument,
                 ...)
 {
   va_list        varg;
   Babl          *babl;
-  int            id         = 0;
-  int            components = 0;
-  const char    *arg        = first_argument;
-  const char    *name       = NULL;
+  int            id            = 0;
+  int            components    = 0;
+  const char    *arg           = first_argument;
+  const char    *assigned_name = NULL;
+  char          *name          = NULL;
   BablComponent *component [BABL_MAX_COMPONENTS];
 
   va_start (varg, first_argument);
@@ -107,7 +122,7 @@ babl_model_new (void *first_argument,
                 if (components >= BABL_MAX_COMPONENTS)
                   {
                     babl_log ("maximum number of components (%i) exceeded for %s",
-                              BABL_MAX_COMPONENTS, name);
+                              BABL_MAX_COMPONENTS, assigned_name);
                   }
                 break;
 
@@ -148,12 +163,12 @@ babl_model_new (void *first_argument,
 
       else if (!strcmp (arg, "name"))
         {
-          name = va_arg (varg, char *);
+          assigned_name = va_arg (varg, char *);
         }
 
       else
         {
-          babl_fatal ("unhandled argument '%s' for babl_model '%s'", arg, name);
+          babl_fatal ("unhandled argument '%s' for babl_model '%s'", arg, assigned_name);
         }
 
       arg = va_arg (varg, char *);
@@ -163,23 +178,30 @@ babl_model_new (void *first_argument,
 
   va_end (varg);
 
-  name = create_name (name, components, component);
+  if (assigned_name)
+    name = babl_strdup(assigned_name);
+  else
+    name = babl_model_create_name (components, component);
 
   babl = babl_db_exist (db, id, name);
-  if (babl)
+  if (id && !babl && babl_db_exist (db, 0, name))
+    babl_fatal ("Trying to reregister BablModel '%s' with different id!", name);
+
+  if (! babl)
     {
-      /* There is an instance already registered by the required id/name,
-       * returning the preexistent one instead.
-       */
-      return babl;
+      babl = model_new (name, id, components, component);
+      babl_db_insert (db, babl);
+      construct_double_format (babl);
+    }
+  else
+    {
+      if (!is_model_duplicate (babl, components, component))
+        babl_fatal ("BablModel '%s' already registered "
+                    "with different components!", name);
     }
 
-  babl = model_new (name, id, components, component);
+  babl_free (name);
 
-  /* Since there is not an already registered instance by the required
-   * id/name, inserting newly created class into database.
-   */
-  babl_db_insert (db, babl);
   return babl;
 }
 
@@ -222,12 +244,17 @@ static Babl *reference_format (void)
 
 static Babl *construct_double_format (Babl *model)
 {
-  void *argument[42 + 1];
+  void *argument[44 + 1];
   int   args = 0;
   int   i;
 
+  if (model == babl_model_from_id (BABL_RGBA))
+    {
+      argument[args++] = "id";
+      argument[args++] = (void*) BABL_RGBA_DOUBLE;
+    }
   argument[args++] = model;
-  argument[args++] = babl_type ("double");
+  argument[args++] = babl_type_from_id (BABL_DOUBLE);
 
   for (i = 0; i < model->model.components; i++)
     {
