@@ -21,6 +21,7 @@
 #define VERTICAL_MIN      1
 #define VERTICAL_MAX      4
 
+#include "config.h"
 #include "babl-internal.h"
 
 static BablSampling sampling_db[(HORIZONTAL_MAX - HORIZONTAL_MIN + 1) *
@@ -41,18 +42,9 @@ babl_sampling (int horizontal,
   return NULL;
 }
 
-
-static int
-each_babl_sampling_destroy (Babl *babl,
-                            void *data)
-{
-  babl_free (babl->sampling.from);
-  return 0;  /* continue iterating */
-}
-
 void
-babl_sampling_each (BablEachFunction each_fun,
-                    void            *user_data)
+babl_sampling_class_for_each (BablEachFunction each_fun,
+                              void            *user_data)
 {
   int horizontal;
   int vertical;
@@ -66,15 +58,8 @@ babl_sampling_each (BablEachFunction each_fun,
       }
 }
 
-
 void
-babl_sampling_destroy (void)
-{
-  babl_sampling_each (each_babl_sampling_destroy, NULL);
-}
-
-void
-babl_sampling_init (void)
+babl_sampling_class_init (void)
 {
   int horizontal;
   int vertical;
