@@ -16,6 +16,7 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include <string.h>
 #include <stdarg.h>
 #include <math.h>
@@ -23,18 +24,15 @@
 #include "babl-internal.h"
 #include "babl-db.h"
 
-
 static int
-each_babl_type_destroy (Babl *babl,
-                        void *data)
+babl_type_destroy (void *data)
 {
+  Babl *babl = data;
   if (babl->type.from_list)
-    babl_list_destroy (babl->type.from_list);
-  babl_free (babl);
-  return 0;  /* continue iterating */
+    babl_free (babl->type.from_list);
+  return 0; 
 }
 
-
 static Babl *
 type_new (const char *name,
           int         id,
@@ -46,6 +44,7 @@ type_new (const char *name,
   babl_assert (bits % 8 == 0);
 
   babl                 = babl_malloc (sizeof (BablType) + strlen (name) + 1);
+  babl_set_destructor (babl, babl_type_destroy);
   babl->instance.name  = (void *) ((char *) babl + sizeof (BablType));
   babl->class_type     = BABL_TYPE;
   babl->instance.id    = id;
@@ -56,6 +55,15 @@ type_new (const char *name,
   return babl;
 }
 
+static int
+is_type_duplicate (Babl *babl, int bits)
+{
+  if (babl->type.bits != bits)
+    return 0;
+
+  return 1;
+}
+
 Babl *
 babl_type_new (void *first_arg,
                ...)
@@ -69,8 +77,8 @@ babl_type_new (void *first_arg,
   long        max        = 255;
   double      min_val    = 0.0;
   double      max_val    = 0.0;
-
-  const char *arg = first_arg;
+  const char *name = first_arg;
+  const char *arg;
 
   va_start (varg, first_arg);
 
@@ -123,22 +131,30 @@ babl_type_new (void *first_arg,
 
       else
         {
-          babl_fatal ("unhandled argument '%s' for format '%s'", arg, first_arg);
+          babl_fatal ("unhandled argument '%s' for format '%s'", arg, name);
         }
     }
 
   va_end (varg);
 
-  babl = babl_db_exist (db, id, first_arg);
+  babl = babl_db_exist (db, id, name);
+  if (id && !babl && babl_db_exist (db, 0, name))
+    babl_fatal ("Trying to reregister BablType '%s' with different id!", name);
+
   if (babl)
     {
       /* There is an instance already registered by the required id/name,
-       * returning the preexistent one instead.
+       * returning the preexistent one instead if it doesn't differ.
        */
+
+      if (!is_type_duplicate (babl, bits))
+        babl_fatal ("BablType '%s' already registered "
+                    "as different type!", name);
+
       return babl;
     }
 
-  babl = type_new (first_arg, id, bits);
+  babl = type_new (name, id, bits);
 
   /* Since there is not an already registered instance by the required
    * id/name, inserting newly created class into database.
@@ -252,4 +268,4 @@ babl_type_is_symmetric (Babl *babl)
   return is_symmetrical;
 }
 
-BABL_CLASS (type)
+BABL_CLASS_IMPLEMENT (type)
